@@ -27,6 +27,8 @@
 #include "config.h"
 
 #include "ggettext.h"
+#include "glibintl.h"
+#include "glib-private.h"
 
 #include "galloca.h"
 #include "gthread.h"
@@ -333,10 +335,11 @@ _g_dgettext_should_translate (void)
        *   - If locale is "C", maybe user calls setlocale(LC_ALL,"") later.
        *     Continue with old behavior of translating.
        */
-      if (0 != strcmp (default_domain, "messages") &&
+      if (!default_domain || !translator_comment || !translate_locale ||
+          (0 != strcmp (default_domain, "messages") &&
           '\0' == *translator_comment &&
           0 != strncmp (translate_locale, "en_", 3) &&
-          0 != strcmp (translate_locale, "C"))
+          0 != strcmp (translate_locale, "C")))
         should_translate = FALSE;
 
       g_once_init_leave (&translate,
@@ -470,16 +473,21 @@ g_dngettext (const gchar *domain,
  * easy-to-use form.
  *
  * In order to use these macros in an application, you must include
- * <filename>glib/gi18n.h</filename>. For use in a library, must include
+ * <filename>glib/gi18n.h</filename>. For use in a library, you must include
  * <filename>glib/gi18n-lib.h</filename> <emphasis>after</emphasis> defining
  * the GETTEXT_PACKAGE macro suitably for your library:
  * |[
  * &num;define GETTEXT_PACKAGE "gtk20"
  * &num;include &lt;glib/gi18n-lib.h&gt;
  * ]|
- * Note that you also have to call setlocale() and textdomain() (as well as
- * bindtextdomain() and bind_textdomain_codeset()) early on in your main()
- * to make gettext() work.
+ * For an application, note that you also have to call bindtextdomain(),
+ * bind_textdomain_codeset(), textdomain() and setlocale() early on in your
+ * main() to make gettext() work.
+ *
+ * For a library, you only have to call bindtextdomain() and
+ * bind_textdomain_codeset() in your initialization function. If your library
+ * doesn't have an initialization function, you can call the functions before
+ * the first translated message.
  *
  * The gettext manual covers details of how to set up message extraction
  * with xgettext.
